@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.domains.c
     ~~~~~~~~~~~~~~~~
 
     The C language domain.
 
-    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,12 +14,19 @@ import string
 from docutils import nodes
 
 from sphinx import addnodes
-from sphinx.roles import XRefRole
-from sphinx.locale import l_, _
-from sphinx.domains import Domain, ObjType
 from sphinx.directives import ObjectDescription
-from sphinx.util.nodes import make_refnode
+from sphinx.domains import Domain, ObjType
+from sphinx.locale import _
+from sphinx.roles import XRefRole
 from sphinx.util.docfields import Field, TypedField
+from sphinx.util.nodes import make_refnode
+
+if False:
+    # For type annotation
+    from typing import Any, Dict, Iterator, List, Tuple  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.builders import Builder  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
 
 
 # RE to split at word boundaries
@@ -39,6 +45,13 @@ c_funcptr_sig_re = re.compile(
         \( (.*) \)         # arguments
         (\s+const)? $      # const specifier
     ''', re.VERBOSE)
+c_funcptr_arg_sig_re = re.compile(
+    r'''^\s*([^(,]+?)      # return type
+        \( ([^()]+) \) \s* # name in parentheses
+        \( (.*) \)         # arguments
+        (\s+const)?        # const specifier
+        \s*(?=$|,)         # end with comma or end of string
+    ''', re.VERBOSE)
 c_funcptr_name_re = re.compile(r'^\(\s*\*\s*(.*?)\s*\)$')
 
 
@@ -48,30 +61,31 @@ class CObject(ObjectDescription):
     """
 
     doc_field_types = [
-        TypedField('parameter', label=l_('Parameters'),
+        TypedField('parameter', label=_('Parameters'),
                    names=('param', 'parameter', 'arg', 'argument'),
                    typerolename='type', typenames=('type',)),
-        Field('returnvalue', label=l_('Returns'), has_arg=False,
+        Field('returnvalue', label=_('Returns'), has_arg=False,
               names=('returns', 'return')),
-        Field('returntype', label=l_('Return type'), has_arg=False,
+        Field('returntype', label=_('Return type'), has_arg=False,
               names=('rtype',)),
     ]
 
     # These C types aren't described anywhere, so don't try to create
     # a cross-reference to them
-    stopwords = set((
+    stopwords = {
         'const', 'void', 'char', 'wchar_t', 'int', 'short',
         'long', 'float', 'double', 'unsigned', 'signed', 'FILE',
         'clock_t', 'time_t', 'ptrdiff_t', 'size_t', 'ssize_t',
         'struct', '_Bool',
-    ))
+    }
 
     def _parse_type(self, node, ctype):
+        # type: (nodes.Element, str) -> None
         # add cross-ref nodes for all words
-        for part in filter(None, wsplit_re.split(ctype)):
+        for part in [_f for _f in wsplit_re.split(ctype) if _f]:
             tnode = nodes.Text(part, part)
-            if part[0] in string.ascii_letters+'_' and \
-                   part not in self.stopwords:
+            if part[0] in string.ascii_letters + '_' and \
+               part not in self.stopwords:
                 pnode = addnodes.pending_xref(
                     '', refdomain='c', reftype='type', reftarget=part,
                     modname=None, classname=None)
@@ -80,7 +94,27 @@ class CObject(ObjectDescription):
             else:
                 node += tnode
 
+    def _parse_arglist(self, arglist):
+        # type: (str) -> Iterator[str]
+        while True:
+            m = c_funcptr_arg_sig_re.match(arglist)
+            if m:
+                yield m.group()
+                arglist = c_funcptr_arg_sig_re.sub('', arglist)
+                if ',' in arglist:
+                    _, arglist = arglist.split(',', 1)
+                else:
+                    break
+            else:
+                if ',' in arglist:
+                    arg, arglist = arglist.split(',', 1)
+                    yield arg
+                else:
+                    yield arglist
+                    break
+
     def handle_signature(self, sig, signode):
+        # type: (str, addnodes.desc_signature) -> str
         """Transform a C signature into RST nodes."""
         # first try the function pointer signature regex, it's more specific
         m = c_funcptr_sig_re.match(sig)
@@ -90,8 +124,9 @@ class CObject(ObjectDescription):
             raise ValueError('no match')
         rettype, name, arglist, const = m.groups()
 
-        signode += addnodes.desc_type('', '')
-        self._parse_type(signode[-1], rettype)
+        desc_type = addnodes.desc_type('', '')
+        signode += desc_type
+        self._parse_type(desc_type, rettype)
         try:
             classname, funcname = name.split('::', 1)
             classname += '::'
@@ -105,14 +140,15 @@ class CObject(ObjectDescription):
         if m:
             name = m.group(1)
 
-        typename = self.env.temp_data.get('c:type')
+        typename = self.env.ref_context.get('c:type')
         if self.name == 'c:member' and typename:
             fullname = typename + '.' + name
         else:
             fullname = name
 
         if not arglist:
-            if self.objtype == 'function':
+            if self.objtype == 'function' or \
+                    self.objtype == 'macro' and sig.rstrip().endswith('()'):
                 # for functions, add an empty parameter list
                 signode += addnodes.desc_parameterlist()
             if const:
@@ -120,21 +156,27 @@ class CObject(ObjectDescription):
             return fullname
 
         paramlist = addnodes.desc_parameterlist()
-        arglist = arglist.replace('`', '').replace('\\ ', '') # remove markup
+        arglist = arglist.replace('`', '').replace('\\ ', '')  # remove markup
         # this messes up function pointer types, but not too badly ;)
-        args = arglist.split(',')
-        for arg in args:
+        for arg in self._parse_arglist(arglist):
             arg = arg.strip()
             param = addnodes.desc_parameter('', '', noemph=True)
             try:
-                ctype, argname = arg.rsplit(' ', 1)
+                m = c_funcptr_arg_sig_re.match(arg)
+                if m:
+                    self._parse_type(param, m.group(1) + '(')
+                    param += nodes.emphasis(m.group(2), m.group(2))
+                    self._parse_type(param, ')(' + m.group(3) + ')')
+                    if m.group(4):
+                        param += addnodes.desc_addname(m.group(4), m.group(4))
+                else:
+                    ctype, argname = arg.rsplit(' ', 1)
+                    self._parse_type(param, ctype)
+                    # separate by non-breaking space in the output
+                    param += nodes.emphasis(' ' + argname, '\xa0' + argname)
             except ValueError:
                 # no argument name given, only the type
                 self._parse_type(param, arg)
-            else:
-                self._parse_type(param, ctype)
-                # separate by non-breaking space in the output
-                param += nodes.emphasis(' '+argname, u'\xa0'+argname)
             paramlist += param
         signode += paramlist
         if const:
@@ -142,6 +184,7 @@ class CObject(ObjectDescription):
         return fullname
 
     def get_index_text(self, name):
+        # type: (str) -> str
         if self.objtype == 'function':
             return _('%s (C function)') % name
         elif self.objtype == 'member':
@@ -156,6 +199,7 @@ class CObject(ObjectDescription):
             return ''
 
     def add_target_and_index(self, name, sig, signode):
+        # type: (str, str, addnodes.desc_signature) -> None
         # for C API items we add a prefix since names are usually not qualified
         # by a module name and so easily clash with e.g. section titles
         targetname = 'c.' + name
@@ -175,31 +219,34 @@ class CObject(ObjectDescription):
         indextext = self.get_index_text(name)
         if indextext:
             self.indexnode['entries'].append(('single', indextext,
-                                              targetname, ''))
+                                              targetname, '', None))
 
     def before_content(self):
+        # type: () -> None
         self.typename_set = False
         if self.name == 'c:type':
             if self.names:
-                self.env.temp_data['c:type'] = self.names[0]
+                self.env.ref_context['c:type'] = self.names[0]
                 self.typename_set = True
 
     def after_content(self):
+        # type: () -> None
         if self.typename_set:
-            self.env.temp_data['c:type'] = None
+            self.env.ref_context.pop('c:type', None)
 
 
 class CXRefRole(XRefRole):
     def process_link(self, env, refnode, has_explicit_title, title, target):
+        # type: (BuildEnvironment, nodes.Element, bool, str, str) -> Tuple[str, str]
         if not has_explicit_title:
-            target = target.lstrip('~') # only has a meaning for the title
+            target = target.lstrip('~')  # only has a meaning for the title
             # if the first character is a tilde, don't display the module/class
             # parts of the contents
             if title[0:1] == '~':
                 title = title[1:]
                 dot = title.rfind('.')
                 if dot != -1:
-                    title = title[dot+1:]
+                    title = title[dot + 1:]
         return title, target
 
 
@@ -208,11 +255,11 @@ class CDomain(Domain):
     name = 'c'
     label = 'C'
     object_types = {
-        'function': ObjType(l_('function'), 'func'),
-        'member':   ObjType(l_('member'),   'member'),
-        'macro':    ObjType(l_('macro'),    'macro'),
-        'type':     ObjType(l_('type'),     'type'),
-        'var':      ObjType(l_('variable'), 'data'),
+        'function': ObjType(_('function'), 'func'),
+        'member':   ObjType(_('member'),   'member'),
+        'macro':    ObjType(_('macro'),    'macro'),
+        'type':     ObjType(_('type'),     'type'),
+        'var':      ObjType(_('variable'), 'data'),
     }
 
     directives = {
@@ -223,7 +270,7 @@ class CDomain(Domain):
         'var':      CObject,
     }
     roles = {
-        'func' :  CXRefRole(fix_parens=True),
+        'func':   CXRefRole(fix_parens=True),
         'member': CXRefRole(),
         'macro':  CXRefRole(),
         'data':   CXRefRole(),
@@ -231,23 +278,60 @@ class CDomain(Domain):
     }
     initial_data = {
         'objects': {},  # fullname -> docname, objtype
-    }
+    }  # type: Dict[str, Dict[str, Tuple[str, Any]]]
 
     def clear_doc(self, docname):
-        for fullname, (fn, _) in self.data['objects'].items():
+        # type: (str) -> None
+        for fullname, (fn, _l) in list(self.data['objects'].items()):
             if fn == docname:
                 del self.data['objects'][fullname]
 
+    def merge_domaindata(self, docnames, otherdata):
+        # type: (List[str], Dict) -> None
+        # XXX check duplicates
+        for fullname, (fn, objtype) in otherdata['objects'].items():
+            if fn in docnames:
+                self.data['objects'][fullname] = (fn, objtype)
+
     def resolve_xref(self, env, fromdocname, builder,
                      typ, target, node, contnode):
+        # type: (BuildEnvironment, str, Builder, str, str, addnodes.pending_xref, nodes.Element) -> nodes.Element  # NOQA
         # strip pointer asterisk
         target = target.rstrip(' *')
+        # becase TypedField can generate xrefs
+        if target in CObject.stopwords:
+            return contnode
         if target not in self.data['objects']:
             return None
         obj = self.data['objects'][target]
         return make_refnode(builder, fromdocname, obj[0], 'c.' + target,
                             contnode, target)
 
+    def resolve_any_xref(self, env, fromdocname, builder, target,
+                         node, contnode):
+        # type: (BuildEnvironment, str, Builder, str, addnodes.pending_xref, nodes.Element) -> List[Tuple[str, nodes.Element]]  # NOQA
+        # strip pointer asterisk
+        target = target.rstrip(' *')
+        if target not in self.data['objects']:
+            return []
+        obj = self.data['objects'][target]
+        return [('c:' + self.role_for_objtype(obj[1]),
+                 make_refnode(builder, fromdocname, obj[0], 'c.' + target,
+                              contnode, target))]
+
     def get_objects(self):
-        for refname, (docname, type) in self.data['objects'].iteritems():
+        # type: () -> Iterator[Tuple[str, str, str, str, str, int]]
+        for refname, (docname, type) in list(self.data['objects'].items()):
             yield (refname, refname, type, docname, 'c.' + refname, 1)
+
+
+def setup(app):
+    # type: (Sphinx) -> Dict[str, Any]
+    app.add_domain(CDomain)
+
+    return {
+        'version': 'builtin',
+        'env_version': 1,
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
