@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     sphinx.ext.autosummary
     ~~~~~~~~~~~~~~~~~~~~~~
@@ -49,27 +48,54 @@
     resolved to a Python object, and otherwise it becomes simple emphasis.
     This can be used as the default role to make links 'smart'.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+import inspect
 import os
+import posixpath
 import re
 import sys
-import inspect
-import posixpath
+import warnings
 from types import ModuleType
+from typing import List, cast
 
-from six import text_type
-from docutils.parsers.rst import directives
-from docutils.statemachine import ViewList
 from docutils import nodes
+from docutils.parsers.rst import directives
+from docutils.parsers.rst.states import RSTStateMachine, Struct, state_classes
+from docutils.statemachine import StringList
 
 import sphinx
 from sphinx import addnodes
-from sphinx.util.compat import Directive
+from sphinx.deprecation import RemovedInSphinx40Warning
+from sphinx.environment.adapters.toctree import TocTree
+from sphinx.ext.autodoc import get_documenters
+from sphinx.ext.autodoc.directive import DocumenterBridge, Options
+from sphinx.ext.autodoc.importer import import_module
+from sphinx.ext.autodoc.mock import mock
+from sphinx.locale import __
 from sphinx.pycode import ModuleAnalyzer, PycodeError
-from sphinx.ext.autodoc import Options
+from sphinx.util import import_object, rst, logging
+from sphinx.util.docutils import (
+    NullReporter, SphinxDirective, SphinxRole, new_document, switch_source_input
+)
+from sphinx.util.matching import Matcher
+
+if False:
+    # For type annotation
+    from typing import Any, Dict, Tuple, Type  # NOQA
+    from docutils.parsers.rst.states import Inliner  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+    from sphinx.environment import BuildEnvironment  # NOQA
+    from sphinx.ext.autodoc import Documenter  # NOQA
+    from sphinx.writers.html import HTMLTranslator  # NOQA
+
+logger = logging.getLogger(__name__)
+
+
+periods_re = re.compile(r'\.(?:\s+)')
+literal_re = re.compile(r'::\s*$')
 
 
 # -- autosummary_toc node ------------------------------------------------------
@@ -79,6 +105,7 @@ class autosummary_toc(nodes.comment):
 
 
 def process_autosummary_toc(app, doctree):
+    # type: (Sphinx, nodes.document) -> None
     """Insert items described in autosummary:: to the TOC tree, but do
     not generate the toctree:: list.
     """
@@ -86,28 +113,31 @@ def process_autosummary_toc(app, doctree):
     crawled = {}
 
     def crawl_toc(node, depth=1):
+        # type: (nodes.Element, int) -> None
         crawled[node] = True
         for j, subnode in enumerate(node):
             try:
                 if (isinstance(subnode, autosummary_toc) and
                         isinstance(subnode[0], addnodes.toctree)):
-                    env.note_toctree(env.docname, subnode[0])
+                    TocTree(env).note(env.docname, subnode[0])
                     continue
             except IndexError:
                 continue
             if not isinstance(subnode, nodes.section):
                 continue
             if subnode not in crawled:
-                crawl_toc(subnode, depth+1)
+                crawl_toc(subnode, depth + 1)
     crawl_toc(doctree)
 
 
 def autosummary_toc_visit_html(self, node):
+    # type: (nodes.NodeVisitor, autosummary_toc) -> None
     """Hide autosummary toctree list in HTML output."""
     raise nodes.SkipNode
 
 
 def autosummary_noop(self, node):
+    # type: (nodes.NodeVisitor, nodes.Node) -> None
     pass
 
 
@@ -118,16 +148,19 @@ class autosummary_table(nodes.comment):
 
 
 def autosummary_table_visit_html(self, node):
+    # type: (HTMLTranslator, autosummary_table) -> None
     """Make the first column of the table non-breaking."""
     try:
-        tbody = node[0][0][-1]
-        for row in tbody:
-            col1_entry = row[0]
-            par = col1_entry[0]
+        table = cast(nodes.table, node[0])
+        tgroup = cast(nodes.tgroup, table[0])
+        tbody = cast(nodes.tbody, tgroup[-1])
+        rows = cast(List[nodes.row], tbody)
+        for row in rows:
+            col1_entry = cast(nodes.entry, row[0])
+            par = cast(nodes.paragraph, col1_entry[0])
             for j, subnode in enumerate(list(par)):
                 if isinstance(subnode, nodes.Text):
-                    new_text = text_type(subnode.astext())
-                    new_text = new_text.replace(u" ", u"\u00a0")
+                    new_text = subnode.astext().replace(" ", "\u00a0")
                     par[j] = nodes.Text(new_text)
     except IndexError:
         pass
@@ -135,12 +168,21 @@ def autosummary_table_visit_html(self, node):
 
 # -- autodoc integration -------------------------------------------------------
 
-class FakeDirective:
-    env = {}
-    genopt = Options()
+# current application object (used in `get_documenter()`).
+_app = None  # type: Sphinx
 
 
-def get_documenter(obj, parent):
+class FakeDirective(DocumenterBridge):
+    def __init__(self):
+        # type: () -> None
+        settings = Struct(tab_width=8)
+        document = Struct(settings=settings)
+        state = Struct(document=document)
+        super().__init__({}, None, Options(), 0, state)  # type: ignore
+
+
+def get_documenter(app, obj, parent):
+    # type: (Sphinx, Any, Any) -> Type[Documenter]
     """Get an autodoc.Documenter class suitable for documenting the given
     object.
 
@@ -148,8 +190,7 @@ def get_documenter(obj, parent):
     another Python object (e.g. a module or a class) to which *obj*
     belongs to.
     """
-    from sphinx.ext.autodoc import AutoDirective, DataDocumenter, \
-        ModuleDocumenter
+    from sphinx.ext.autodoc import DataDocumenter, ModuleDocumenter
 
     if inspect.ismodule(obj):
         # ModuleDocumenter.can_document_member always returns False
@@ -157,7 +198,7 @@ def get_documenter(obj, parent):
 
     # Construct a fake documenter for *parent*
     if parent is not None:
-        parent_doc_cls = get_documenter(parent, None)
+        parent_doc_cls = get_documenter(app, parent, None)
     else:
         parent_doc_cls = ModuleDocumenter
 
@@ -167,7 +208,7 @@ def get_documenter(obj, parent):
         parent_doc = parent_doc_cls(FakeDirective(), "")
 
     # Get the corrent documenter class for *obj*
-    classes = [cls for cls in AutoDirective._registry.values()
+    classes = [cls for cls in get_documenters(app).values()
                if cls.can_document_member(obj, '', False, parent_doc)]
     if classes:
         classes.sort(key=lambda cls: cls.priority)
@@ -178,7 +219,7 @@ def get_documenter(obj, parent):
 
 # -- .. autosummary:: ----------------------------------------------------------
 
-class Autosummary(Directive):
+class Autosummary(SphinxDirective):
     """
     Pretty table containing short signatures and summaries of functions etc.
 
@@ -195,15 +236,10 @@ class Autosummary(Directive):
         'template': directives.unchanged,
     }
 
-    def warn(self, msg):
-        self.warnings.append(self.state.document.reporter.warning(
-            msg, line=self.lineno))
-
     def run(self):
-        self.env = env = self.state.document.settings.env
-        self.genopt = Options()
-        self.warnings = []
-        self.result = ViewList()
+        # type: () -> List[nodes.Node]
+        self.bridge = DocumenterBridge(self.env, self.state.document.reporter,
+                                       Options(), self.lineno, self.state)
 
         names = [x.strip().split()[0] for x in self.content
                  if x.strip() and re.search(r'^[~a-zA-Z_]', x.strip()[0])]
@@ -211,16 +247,19 @@ class Autosummary(Directive):
         nodes = self.get_table(items)
 
         if 'toctree' in self.options:
-            dirname = posixpath.dirname(env.docname)
+            dirname = posixpath.dirname(self.env.docname)
 
             tree_prefix = self.options['toctree'].strip()
             docnames = []
+            excluded = Matcher(self.config.exclude_patterns)
             for name, sig, summary, real_name in items:
                 docname = posixpath.join(tree_prefix, real_name)
                 docname = posixpath.normpath(posixpath.join(dirname, docname))
-                if docname not in env.found_docs:
-                    self.warn('toctree references unknown document %r'
-                              % docname)
+                if docname not in self.env.found_docs:
+                    if excluded(self.env.doc2path(docname, None)):
+                        logger.warning(__('toctree references excluded document %r'), docname)
+                    else:
+                        logger.warning(__('toctree references unknown document %r'), docname)
                 docnames.append(docname)
 
             tocnode = addnodes.toctree()
@@ -229,20 +268,18 @@ class Autosummary(Directive):
             tocnode['maxdepth'] = -1
             tocnode['glob'] = None
 
-            tocnode = autosummary_toc('', '', tocnode)
-            nodes.append(tocnode)
+            nodes.append(autosummary_toc('', '', tocnode))
 
-        return self.warnings + nodes
+        return nodes
 
     def get_items(self, names):
+        # type: (List[str]) -> List[Tuple[str, str, str, str]]
         """Try to import the given names, and return a list of
         ``[(name, signature, summary_string, real_name), ...]``.
         """
-        env = self.state.document.settings.env
+        prefixes = get_import_prefixes_from_env(self.env)
 
-        prefixes = get_import_prefixes_from_env(env)
-
-        items = []
+        items = []  # type: List[Tuple[str, str, str, str]]
 
         max_item_chars = 50
 
@@ -253,30 +290,32 @@ class Autosummary(Directive):
                 display_name = name.split('.')[-1]
 
             try:
-                real_name, obj, parent, modname = import_by_name(name, prefixes=prefixes)
+                with mock(self.config.autosummary_mock_imports):
+                    real_name, obj, parent, modname = import_by_name(name, prefixes=prefixes)
             except ImportError:
-                self.warn('failed to import %s' % name)
+                logger.warning(__('failed to import %s'), name)
                 items.append((name, '', '', name))
                 continue
 
-            self.result = ViewList()  # initialize for each documenter
+            self.bridge.result = StringList()  # initialize for each documenter
             full_name = real_name
             if not isinstance(obj, ModuleType):
                 # give explicitly separated module name, so that members
                 # of inner classes can be documented
-                full_name = modname + '::' + full_name[len(modname)+1:]
+                full_name = modname + '::' + full_name[len(modname) + 1:]
             # NB. using full_name here is important, since Documenters
             #     handle module prefixes slightly differently
-            documenter = get_documenter(obj, parent)(self, full_name)
+            doccls = get_documenter(self.env.app, obj, parent)
+            documenter = doccls(self.bridge, full_name)
             if not documenter.parse_name():
-                self.warn('failed to parse name %s' % real_name)
+                logger.warning(__('failed to parse name %s'), real_name)
                 items.append((display_name, '', '', real_name))
                 continue
             if not documenter.import_object():
-                self.warn('failed to import object %s' % real_name)
+                logger.warning(__('failed to import object %s'), real_name)
                 items.append((display_name, '', '', real_name))
                 continue
-            if not documenter.check_module():
+            if documenter.options.members and not documenter.check_module():
                 continue
 
             # try to also get a source code analyzer for attribute docs
@@ -287,57 +326,41 @@ class Autosummary(Directive):
                 # be cached anyway)
                 documenter.analyzer.find_attr_docs()
             except PycodeError as err:
-                documenter.env.app.debug(
-                    '[autodoc] module analyzer failed: %s', err)
+                logger.debug('[autodoc] module analyzer failed: %s', err)
                 # no source file -- e.g. for builtin and C modules
                 documenter.analyzer = None
 
             # -- Grab the signature
 
-            sig = documenter.format_signature()
+            try:
+                sig = documenter.format_signature(show_annotation=False)
+            except TypeError:
+                # the documenter does not support ``show_annotation`` option
+                sig = documenter.format_signature()
+
             if not sig:
                 sig = ''
             else:
                 max_chars = max(10, max_item_chars - len(display_name))
                 sig = mangle_signature(sig, max_chars=max_chars)
-                sig = sig.replace('*', r'\*')
 
             # -- Grab the summary
 
             documenter.add_content(None)
-            doc = list(documenter.process_doc([self.result.data]))
-
-            while doc and not doc[0].strip():
-                doc.pop(0)
-
-            # If there's a blank line, then we can assume the first sentence /
-            # paragraph has ended, so anything after shouldn't be part of the
-            # summary
-            for i, piece in enumerate(doc):
-                if not piece.strip():
-                    doc = doc[:i]
-                    break
-
-            # Try to find the "first sentence", which may span multiple lines
-            m = re.search(r"^([A-Z].*?\.)(?:\s|$)", " ".join(doc).strip())
-            if m:
-                summary = m.group(1).strip()
-            elif doc:
-                summary = doc[0].strip()
-            else:
-                summary = ''
+            summary = extract_summary(self.bridge.result.data[:], self.state.document)
 
             items.append((display_name, sig, summary, real_name))
 
         return items
 
     def get_table(self, items):
+        # type: (List[Tuple[str, str, str, str]]) -> List[nodes.Node]
         """Generate a proper list of table nodes for autosummary:: directive.
 
         *items* is a list produced by :meth:`get_items`.
         """
         table_spec = addnodes.tabular_col_spec()
-        table_spec['spec'] = 'll'
+        table_spec['spec'] = r'\X{1}{2}\X{1}{2}'
 
         table = autosummary_table('')
         real_table = nodes.table('', classes=['longtable'])
@@ -350,24 +373,27 @@ class Autosummary(Directive):
         group.append(body)
 
         def append_row(*column_texts):
+            # type: (str) -> None
             row = nodes.row('')
+            source, line = self.state_machine.get_source_and_line()
             for text in column_texts:
                 node = nodes.paragraph('')
-                vl = ViewList()
-                vl.append(text, '<autosummary>')
-                self.state.nested_parse(vl, 0, node)
-                try:
-                    if isinstance(node[0], nodes.paragraph):
-                        node = node[0]
-                except IndexError:
-                    pass
-                row.append(nodes.entry('', node))
+                vl = StringList()
+                vl.append(text, '%s:%d:<autosummary>' % (source, line))
+                with switch_source_input(self.state, vl):
+                    self.state.nested_parse(vl, 0, node)
+                    try:
+                        if isinstance(node[0], nodes.paragraph):
+                            node = node[0]
+                    except IndexError:
+                        pass
+                    row.append(nodes.entry('', node))
             body.append(row)
 
         for name, sig, summary, real_name in items:
             qualifier = 'obj'
             if 'nosignatures' not in self.options:
-                col1 = ':%s:`%s <%s>`\ %s' % (qualifier, name, real_name, sig)
+                col1 = ':%s:`%s <%s>`\\ %s' % (qualifier, name, real_name, rst.escape(sig))
             else:
                 col1 = ':%s:`%s <%s>`' % (qualifier, name, real_name)
             col2 = summary
@@ -375,21 +401,69 @@ class Autosummary(Directive):
 
         return [table_spec, table]
 
+    def warn(self, msg):
+        # type: (str) -> None
+        warnings.warn('Autosummary.warn() is deprecated',
+                      RemovedInSphinx40Warning, stacklevel=2)
+        logger.warning(msg)
+
+    @property
+    def genopt(self):
+        # type: () -> Options
+        warnings.warn('Autosummary.genopt is deprecated',
+                      RemovedInSphinx40Warning, stacklevel=2)
+        return self.bridge.genopt
+
+    @property
+    def warnings(self):
+        # type: () -> List[nodes.Node]
+        warnings.warn('Autosummary.warnings is deprecated',
+                      RemovedInSphinx40Warning, stacklevel=2)
+        return []
+
+    @property
+    def result(self):
+        # type: () -> StringList
+        warnings.warn('Autosummary.result is deprecated',
+                      RemovedInSphinx40Warning, stacklevel=2)
+        return self.bridge.result
+
+
+def strip_arg_typehint(s):
+    # type: (str) -> str
+    """Strip a type hint from argument definition."""
+    return s.split(':')[0].strip()
+
 
 def mangle_signature(sig, max_chars=30):
+    # type: (str, int) -> str
     """Reformat a function signature to a more compact form."""
-    s = re.sub(r"^\((.*)\)$", r"\1", sig).strip()
+    # Strip return type annotation
+    s = re.sub(r"\)\s*->\s.*$", ")", sig)
 
-    # Strip strings (which can contain things that confuse the code below)
-    s = re.sub(r"\\\\", "", s)
-    s = re.sub(r"\\'", "", s)
-    s = re.sub(r"'[^']*'", "", s)
+    # Remove parenthesis
+    s = re.sub(r"^\((.*)\)$", r"\1", s).strip()
+
+    # Strip literals (which can contain things that confuse the code below)
+    s = re.sub(r"\\\\", "", s)      # escaped backslash (maybe inside string)
+    s = re.sub(r"\\'", "", s)       # escaped single quote
+    s = re.sub(r'\\"', "", s)       # escaped double quote
+    s = re.sub(r"'[^']*'", "", s)   # string literal (w/ single quote)
+    s = re.sub(r'"[^"]*"', "", s)   # string literal (w/ double quote)
+
+    # Strip complex objects (maybe default value of arguments)
+    while re.search(r'\([^)]*\)', s):   # contents of parenthesis (ex. NamedTuple(attr=...))
+        s = re.sub(r'\([^)]*\)', '', s)
+    while re.search(r'<[^>]*>', s):     # contents of angle brackets (ex. <object>)
+        s = re.sub(r'<[^>]*>', '', s)
+    while re.search(r'{[^}]*}', s):     # contents of curly brackets (ex. dict)
+        s = re.sub(r'{[^}]*}', '', s)
 
     # Parse the signature to arguments + options
-    args = []
-    opts = []
+    args = []  # type: List[str]
+    opts = []  # type: List[str]
 
-    opt_re = re.compile(r"^(.*, |)([a-zA-Z0-9_*]+)=")
+    opt_re = re.compile(r"^(.*, |)([a-zA-Z0-9_*]+)\s*=\s*")
     while s:
         m = opt_re.search(s)
         if not m:
@@ -400,19 +474,76 @@ def mangle_signature(sig, max_chars=30):
         opts.insert(0, m.group(2))
         s = m.group(1)[:-2]
 
+    # Strip typehints
+    for i, arg in enumerate(args):
+        args[i] = strip_arg_typehint(arg)
+
+    for i, opt in enumerate(opts):
+        opts[i] = strip_arg_typehint(opt)
+
     # Produce a more compact signature
-    sig = limited_join(", ", args, max_chars=max_chars-2)
+    sig = limited_join(", ", args, max_chars=max_chars - 2)
     if opts:
         if not sig:
-            sig = "[%s]" % limited_join(", ", opts, max_chars=max_chars-4)
+            sig = "[%s]" % limited_join(", ", opts, max_chars=max_chars - 4)
         elif len(sig) < max_chars - 4 - 2 - 3:
             sig += "[, %s]" % limited_join(", ", opts,
-                                           max_chars=max_chars-len(sig)-4-2)
+                                           max_chars=max_chars - len(sig) - 4 - 2)
 
-    return u"(%s)" % sig
+    return "(%s)" % sig
+
+
+def extract_summary(doc, document):
+    # type: (List[str], Any) -> str
+    """Extract summary from docstring."""
+
+    # Skip a blank lines at the top
+    while doc and not doc[0].strip():
+        doc.pop(0)
+
+    # If there's a blank line, then we can assume the first sentence /
+    # paragraph has ended, so anything after shouldn't be part of the
+    # summary
+    for i, piece in enumerate(doc):
+        if not piece.strip():
+            doc = doc[:i]
+            break
+
+    if doc == []:
+        return ''
+
+    # parse the docstring
+    state_machine = RSTStateMachine(state_classes, 'Body')
+    node = new_document('', document.settings)
+    node.reporter = NullReporter()
+    state_machine.run(doc, node)
+
+    if not isinstance(node[0], nodes.paragraph):
+        # document starts with non-paragraph: pick up the first line
+        summary = doc[0].strip()
+    else:
+        # Try to find the "first sentence", which may span multiple lines
+        sentences = periods_re.split(" ".join(doc))
+        if len(sentences) == 1:
+            summary = sentences[0].strip()
+        else:
+            summary = ''
+            while sentences:
+                summary += sentences.pop(0) + '.'
+                node[:] = []
+                state_machine.run([summary], node)
+                if not node.traverse(nodes.system_message):
+                    # considered as that splitting by period does not break inline markups
+                    break
+
+    # strip literal notation mark ``::`` from tail of summary
+    summary = literal_re.sub('.', summary)
+
+    return summary
 
 
 def limited_join(sep, items, max_chars=30, overflow_marker="..."):
+    # type: (str, List[str], int, str) -> str
     """Join a number of strings to one, limiting the length to *max_chars*.
 
     If the string overflows this limit, replace the last fitting item by
@@ -439,11 +570,12 @@ def limited_join(sep, items, max_chars=30, overflow_marker="..."):
 # -- Importing items -----------------------------------------------------------
 
 def get_import_prefixes_from_env(env):
+    # type: (BuildEnvironment) -> List[str]
     """
     Obtain current Python import prefixes (for `import_by_name`)
     from ``document.env``
     """
-    prefixes = [None]
+    prefixes = [None]  # type: List[str]
 
     currmodule = env.ref_context.get('py:module')
     if currmodule:
@@ -460,6 +592,7 @@ def get_import_prefixes_from_env(env):
 
 
 def import_by_name(name, prefixes=[None]):
+    # type: (str, List[str]) -> Tuple[str, Any, Any, str]
     """Import a Python object that has the given *name*, under one of the
     *prefixes*.  The first name that succeeds is used.
     """
@@ -478,6 +611,7 @@ def import_by_name(name, prefixes=[None]):
 
 
 def _import_by_name(name):
+    # type: (str) -> Tuple[Any, Any, str]
     """Import a Python object given its full name."""
     try:
         name_parts = name.split('.')
@@ -486,8 +620,7 @@ def _import_by_name(name):
         modname = '.'.join(name_parts[:-1])
         if modname:
             try:
-                __import__(modname)
-                mod = sys.modules[modname]
+                mod = import_module(modname)
                 return getattr(mod, name_parts[-1]), mod, modname
             except (ImportError, IndexError, AttributeError):
                 pass
@@ -495,13 +628,14 @@ def _import_by_name(name):
         # ... then as MODNAME, MODNAME.OBJ1, MODNAME.OBJ1.OBJ2, ...
         last_j = 0
         modname = None
-        for j in reversed(range(1, len(name_parts)+1)):
+        for j in reversed(range(1, len(name_parts) + 1)):
             last_j = j
             modname = '.'.join(name_parts[:j])
             try:
-                __import__(modname)
+                import_module(modname)
             except ImportError:
                 continue
+
             if modname in sys.modules:
                 break
 
@@ -520,29 +654,81 @@ def _import_by_name(name):
 
 # -- :autolink: (smart default role) -------------------------------------------
 
-def autolink_role(typ, rawtext, etext, lineno, inliner,
-                  options={}, content=[]):
+def autolink_role(typ, rawtext, etext, lineno, inliner, options={}, content=[]):
+    # type: (str, str, str, int, Inliner, Dict, List[str]) -> Tuple[List[nodes.Node], List[nodes.system_message]]  # NOQA
     """Smart linking role.
 
     Expands to ':obj:`text`' if `text` is an object that can be imported;
     otherwise expands to '*text*'.
     """
+    warnings.warn('autolink_role() is deprecated.', RemovedInSphinx40Warning)
     env = inliner.document.settings.env
-    r = env.get_domain('py').role('obj')(
-        'obj', rawtext, etext, lineno, inliner, options, content)
-    pnode = r[0][0]
+    pyobj_role = env.get_domain('py').role('obj')
+    objects, msg = pyobj_role('obj', rawtext, etext, lineno, inliner, options, content)
+    if msg != []:
+        return objects, msg
 
+    assert len(objects) == 1
+    pending_xref = cast(addnodes.pending_xref, objects[0])
     prefixes = get_import_prefixes_from_env(env)
     try:
-        name, obj, parent, modname = import_by_name(pnode['reftarget'], prefixes)
+        name, obj, parent, modname = import_by_name(pending_xref['reftarget'], prefixes)
     except ImportError:
-        content = pnode[0]
-        r[0][0] = nodes.emphasis(rawtext, content[0].astext(),
-                                 classes=content['classes'])
-    return r
+        literal = cast(nodes.literal, pending_xref[0])
+        objects[0] = nodes.emphasis(rawtext, literal.astext(), classes=literal['classes'])
+
+    return objects, msg
+
+
+class AutoLink(SphinxRole):
+    """Smart linking role.
+
+    Expands to ':obj:`text`' if `text` is an object that can be imported;
+    otherwise expands to '*text*'.
+    """
+    def run(self):
+        # type: () -> Tuple[List[nodes.Node], List[nodes.system_message]]
+        pyobj_role = self.env.get_domain('py').role('obj')
+        objects, errors = pyobj_role('obj', self.rawtext, self.text, self.lineno,
+                                     self.inliner, self.options, self.content)
+        if errors:
+            return objects, errors
+
+        assert len(objects) == 1
+        pending_xref = cast(addnodes.pending_xref, objects[0])
+        try:
+            # try to import object by name
+            prefixes = get_import_prefixes_from_env(self.env)
+            import_by_name(pending_xref['reftarget'], prefixes)
+        except ImportError:
+            literal = cast(nodes.literal, pending_xref[0])
+            objects[0] = nodes.emphasis(self.rawtext, literal.astext(),
+                                        classes=literal['classes'])
+
+        return objects, errors
+
+
+def get_rst_suffix(app):
+    # type: (Sphinx) -> str
+    def get_supported_format(suffix):
+        # type: (str) -> Tuple[str, ...]
+        parser_class = app.registry.get_source_parsers().get(suffix)
+        if parser_class is None:
+            return ('restructuredtext',)
+        if isinstance(parser_class, str):
+            parser_class = import_object(parser_class, 'source parser')
+        return parser_class.supported
+
+    suffix = None  # type: str
+    for suffix in app.config.source_suffix:
+        if 'restructuredtext' in get_supported_format(suffix):
+            return suffix
+
+    return None
 
 
 def process_generate_options(app):
+    # type: (Sphinx) -> None
     genfiles = app.config.autosummary_generate
 
     if genfiles and not hasattr(genfiles, '__len__'):
@@ -555,16 +741,26 @@ def process_generate_options(app):
 
     from sphinx.ext.autosummary.generate import generate_autosummary_docs
 
-    ext = app.config.source_suffix[0]
-    genfiles = [genfile + (not genfile.endswith(ext) and ext or '')
+    ext = list(app.config.source_suffix)
+    genfiles = [genfile + (not genfile.endswith(tuple(ext)) and ext[0] or '')
                 for genfile in genfiles]
 
-    generate_autosummary_docs(genfiles, builder=app.builder,
-                              warn=app.warn, info=app.info, suffix=ext,
-                              base_path=app.srcdir)
+    suffix = get_rst_suffix(app)
+    if suffix is None:
+        logger.warning(__('autosummary generats .rst files internally. '
+                          'But your source_suffix does not contain .rst. Skipped.'))
+        return
+
+    imported_members = app.config.autosummary_imported_members
+    with mock(app.config.autosummary_mock_imports):
+        generate_autosummary_docs(genfiles, builder=app.builder,
+                                  warn=logger.warning, info=logger.info,
+                                  suffix=suffix, base_path=app.srcdir,
+                                  app=app, imported_members=imported_members)
 
 
 def setup(app):
+    # type: (Sphinx) -> Dict[str, Any]
     # I need autodoc
     app.setup_extension('sphinx.ext.autodoc')
     app.add_node(autosummary_toc,
@@ -580,8 +776,12 @@ def setup(app):
                  man=(autosummary_noop, autosummary_noop),
                  texinfo=(autosummary_noop, autosummary_noop))
     app.add_directive('autosummary', Autosummary)
-    app.add_role('autolink', autolink_role)
+    app.add_role('autolink', AutoLink())
     app.connect('doctree-read', process_autosummary_toc)
     app.connect('builder-inited', process_generate_options)
-    app.add_config_value('autosummary_generate', [], True)
+    app.add_config_value('autosummary_generate', [], True, [bool])
+    app.add_config_value('autosummary_mock_imports',
+                         lambda config: config.autodoc_mock_imports, 'env')
+    app.add_config_value('autosummary_imported_members', [], False, [bool])
+
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
